#ifndef MRK_BUF
#define MRK_BUF

#include <stdlib.h>

#include "mrk/common.h"

/**
 * Wrapper around a nul-terminated char buffer that grows as needed
 */
typedef struct mrk_buf {
  char *s;
  size_t len;
  size_t cap;
} mrk_buf;

/**
 * Initialize a (usually stack-allocated) buffer
 */
mrk_err mrk_buf_init(mrk_buf *buf);

/**
 * Double the buffer's internal capacity until it has enough
 * room for the additional elements.
 */
mrk_err mrk_buf_expand(mrk_buf *buf, size_t required);

/**
 * Shrink the internal buffer so that its capacity equals the stored length.
 */
mrk_err mrk_buf_compact(mrk_buf *buf);

/**
 * Append the given nul-terminated string to the stored string.
 */
mrk_err mrk_buf_append(mrk_buf *buf, const char *s);

/**
 * Append the char buffer with the given length to the stored string.
 */
mrk_err mrk_buf_append_n(mrk_buf *buf, const char *s, size_t len);

#endif
